package numeroaleatorio3;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Esta classe representa o processamento que deve ser feito quando uma
 * única requisição for feita a um servidor. A classe herda de Thread para que
 * várias instâncias dela possam ser executadas concorrentemente.
 */
public class ServicoDeNumerosAleatorios extends Thread {
    // Precisamos armazenar o socket correspondente à conexão.

    private Socket conexão = null;

    /**
     * O construtor desta classe recebe como argumento uma instância de Socket
     * e o armazena em um campo da classe.
     */
    public ServicoDeNumerosAleatorios(Socket conn) {
        conexão = conn;
    }

    /**
     * Este método executa a rotina de atendimento a uma conexão com o servidor.
     */
    public void run() {
        try {
            // Criamos uma stream para receber valores nativos, usando a stream de entrada
            // associado à conexão.
            DataInputStream entrada =
                    new DataInputStream(conexão.getInputStream());
            // Criamos uma stream para enviar valores nativos, usando a stream de saída
            // associado à conexão.
            DataOutputStream saída =
                    new DataOutputStream(conexão.getOutputStream());
            // Lemos do cliente quantos números ele deseja.
            int quantidade = entrada.readInt();

            // Pausa o processamento com um cálculo inútil.
            for (int sleep = 0; sleep < 100000000; sleep++) {
                double x = Math.sqrt(Math.sqrt(Math.sqrt(sleep)));
            }

            // Enviamos para o cliente os números.
            for (int q = 0; q < quantidade; q++) {
                double rand = Math.random();
                saída.writeDouble(rand);
                // Para demonstrar que realmente funciona, vamos imprimir um log.
                System.out.println("Acabo de enviar o número " + rand + " para o cliente "
                        + conexão.getRemoteSocketAddress());
                try {
                    sleep(500);
                } catch (InterruptedException ex) {
                    Logger.getLogger(ServicoDeNumerosAleatorios.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            // Ao terminar de atender a requisição, fechamos as streams.
            entrada.close();
            saída.close();
            // Fechamos também a conexão.
            conexão.close();
        } // Se houve algum erro de entrada ou saída...
        catch (IOException e) {
            System.out.println("Erro atendendo a uma conexão !");
        }
    }
}
