package arquivos;

import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.net.UnknownHostException;

/**
 * Esta classe implementa um cliente simples para o servidor de arquivos.
 * A principal diferença entre este cliente e um cliente simples de texto
 * (telnet, por exemplo) é que existe uma ordem no protocolo de comunicação
 * entre cliente-servidor que pede que em determinado momento, ao invés de
 * receber textos para mostrar no terminal, o cliente deve receber dados para
 * armazenar em um arquivo.
 */
public class ClienteDeArquivos {

    public static void main(String[] args) {
        String servidor = "localhost";
        // Tentamos fazer a conexão...
        try {
            Socket conexão = new Socket(servidor, 2048);
            // A partir do socket podemos obter um InputStream, a partir deste um
            // InputStreamReader e partir deste, um BufferedReader.
            DataInputStream entrada =
                    new DataInputStream(conexão.getInputStream());
            // A partir do socket podemos obter um OutputStream, a partir deste um
            // OutputStreamWriter e partir deste, um BufferedWriter.
            BufferedWriter saída =
                    new BufferedWriter(
                    new OutputStreamWriter(conexão.getOutputStream()));
            // Lemos a lista de arquivos do servidor. Sabemos que vamos ler um
            // número desconhecido de linhas do servidor. Usamos um laço que
            // lê linhas enquanto existirem dados a ser lidos. O servidor enviará
            // a string ##### quando não houverem mais dados.
            while (true) {
                String linha = entrada.readUTF();
                if (linha.equals("#####")) {
                    break; // código do servidor: acabaram as linhas.
                }
                System.out.println(linha);
            }
            // Lemos o nome do arquivo que desejamos do teclado e o
            // enviamos para o servidor.
            String arquivo = Keyboard.readString();
            saída.write(arquivo);
            saída.newLine();
            saída.flush();
            // O servidor envia para o cliente mais uma informação como texto.
            String linha = entrada.readUTF();
            System.out.println(linha);
            // Abrimos o arquivo local. Vamos dar um nome diferente para ele só
            // para não complicar a vida de quem estiver rodando cliente e servidor
            // no mesmo computador.
            DataOutputStream saídaParaArquivo =
                    new DataOutputStream(new FileOutputStream("_" + arquivo));
            // A partir deste momento o servidor vai nos enviar bytes. Lemos todos os
            // bytes enviados do servidor, gravando-os em um arquivo local.
            // Para maior eficiência, vamos ler os dados em blocos de 25 bytes.
            byte[] array = new byte[25];
            while (true) {
                int resultado = entrada.read(array, 0, 25);
                if (resultado == -1) {
                    break;
                }
                // Escrevemos somente os bytes lidos.
                saídaParaArquivo.write(array, 0, resultado);
            }
            // Fechamos as streams e a conexão.
            saída.close();
            entrada.close();
            conexão.close();
        } // Se houve problemas com o nome do host...
        catch (UnknownHostException e) {
            System.out.println("O servidor não existe ou está fora do ar.");
        } // Se houve problemas genéricos de entrada e saída...
        catch (EOFException e) {
            System.out.println("Erro de EOF.");
        } // Se houve problemas genéricos de entrada e saída...
        catch (IOException e) {
            System.out.println("Erro de entrada e saída.");
        }
    }
}
