package daytime;

import java.io.DataOutputStream;
import java.io.IOException;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.Locale;

/**
 * Esta classe implementa um servidor simples que fica aguardando conexoes de
 * clientes. Quando uma conexao e solicitada, o servidor recebe do cliente uma
 * string, a inverte e envia este resuldado para o cliente, fechando a conexao.
 */
public class ServidorDeDaytime {
    // Metodo que permite a execucao da classe.

    public static void main(String[] args) {
        ServerSocket servidor;
        try {
            // Criamos a instancia de ServerSocket que respondera por solicitacoes
            // e porta 10101.
            servidor = new ServerSocket(13);
            // O servidor aguarda "para sempre" as conexoes.
            while (true) {
		System.out.println("processando...");
                // Quando uma conexao e feita,
                Socket conexao = servidor.accept();
                // ... o servidor a processa.
                processaConexao(conexao);

            }
        } // Pode ser que a porta 10101 ja esteja em uso !
        catch (BindException e) {
            System.out.println("Porta ja em uso.");
        } // Pode ser que tenhamos um erro qualquer de entrada ou saida.
        catch (IOException e) {
            System.out.println("Erro de entrada ou saida.");
        }
    }


	private static void processaConexao(Socket conexao) {
        try {
            // Criamos uma stream para enviar valores nativos, usando a stream de saida
            // associado a conexao.
            DataOutputStream saida = new DataOutputStream(conexao.getOutputStream());
            Locale locale = new Locale("pt","BR");
            GregorianCalendar calendar = new GregorianCalendar();
            SimpleDateFormat formatador = new SimpleDateFormat("dd' de 'MMMMM' de 'yyyy' - 'HH':'mm'h'",locale);
            String data = formatador.format(calendar.getTime()).toString();
            saida.writeChars(data);
            saida.flush();
            // Para demonstrar que realmente funciona, vamos imprimir um log.
            System.out.println("Acabo de enviar a data " + data + " para o cliente "
                    + conexao.getRemoteSocketAddress());
            // Ao terminar de atender a requisicao, fechamos a stream de saida.
            saida.close();
            // Fechamos tambem a conexao.
            conexao.close();
        } // Se houve algum erro de entrada ou saida...
        catch (IOException e) {
            System.out.println("Erro atendendo a uma conexao !");
        }
	}
}
