package echo;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Esta classe implementa um servidor simples que fica aguardando conexões de
 * clientes. Quando uma conexão é solicitada, o servidor recebe do cliente uma
 * string, a inverte e envia este resuldado para o cliente, fechando a conexão.
 */
public class ServidorDeEcho {
    // Método que permite a execução da classe.

    public static void main(String[] args) {
        ServerSocket servidor;
        try {
            // Criamos a instância de ServerSocket que responderá por solicitações
            // à porta 10101.
            System.out.println("SERVIDOR STRING");
            servidor = new ServerSocket(10101);
            // O servidor aguarda "para sempre" as conexões.
            while (true) {
                System.out.println("processando...");
                // Quando uma conexão é feita,
                Socket conexão = servidor.accept();
                // ... o servidor a processa.
                processaConexão(conexão);
            }
        } // Pode ser que a porta 10101 já esteja em uso !
        catch (BindException e) {
            System.out.println("Porta já em uso.");
        } // Pode ser que tenhamos um erro qualquer de entrada ou saída.
        catch (IOException e) {
            System.out.println("Erro de entrada ou saída.");
        }
    }
    // Este método atende a uma conexão feita a este servidor.

    private static void processaConexão(Socket conexão) {
        try {
            // Criamos uma stream para receber strings, usando a stream de entrada
            // associado à conexão.
            BufferedReader entrada =
                    new BufferedReader(new InputStreamReader(conexão.getInputStream()));
            // Criamos uma stream para enviar strings, usando a stream de saída
            // associado à conexão.
            BufferedWriter saída =
                    new BufferedWriter(new OutputStreamWriter(conexão.getOutputStream()));
            // Lemos a string que o cliente quer inverter.
            String str = entrada.readLine();
            
            // Enviamos a string invertida ao cliente.
            saída.write(str);
            saída.newLine();
            saída.flush();
            // Ao terminar de atender a requisição, fechamos as streams de entrada e saída.
            entrada.close();
            saída.close();
            // Fechamos também a conexão.
            conexão.close();
        } // Se houve algum erro de entrada ou saída...
        catch (IOException e) {
            System.out.println("Erro atendendo a uma conexão !");
        }
    }
}
