package numerosaleatorios;

import java.io.DataOutputStream;
import java.io.IOException;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Esta classe implementa um servidor simples que fica aguardando conexões de
 * clientes. Quando uma conexão é solicitada, o servidor envia para o cliente
 * um número aleatório e fecha a conexão.
 */
public class ServidorDeNumerosAleatorios {
    // Método que permite a execução da classe.

    public static void main(String[] args) {
        ServerSocket servidor;
        try {
            // Criamos a instância de ServerSocket que responderá por solicitações
            // à porta 9999.
            servidor = new ServerSocket(9999);
            // O servidor aguarda "para sempre" as conexões.
            while (true) {
                // Quando uma conexão é feita,
                Socket conexão = servidor.accept();
                // ... o servidor a processa.
                processaConexão(conexão);
            }
        } // Pode ser que a porta 9999 já esteja em uso !
        catch (BindException e) {
            System.out.println("Porta já em uso.");
        } // Pode ser que tenhamos um erro qualquer de entrada ou saída.
        catch (IOException e) {
            System.out.println("Erro de entrada ou saída.");
        }
    }

    // Este método atende a uma conexão feita a este servidor.
    private static void processaConexão(Socket conexão) {
        try {
            // Criamos uma stream para enviar valores nativos, usando a stream de saída
            // associado à conexão.
            DataOutputStream saída =
                    new DataOutputStream(conexão.getOutputStream());
            double rand = Math.random();
            // Pausa o processamento com um cálculo inútil.
            for (int sleep = 0; sleep < 100000000; sleep++) {
                double x = Math.sqrt(Math.sqrt(Math.sqrt(sleep)));
            }
            saída.writeDouble(rand);
            // Para demonstrar que realmente funciona, vamos imprimir um log.
            System.out.println("Acabo de enviar o número " + rand + " para o cliente "
                    + conexão.getRemoteSocketAddress());
            // Ao terminar de atender a requisição, fechamos a stream de saída.
            saída.close();
            // Fechamos também a conexão.
            conexão.close();
        } // Se houve algum erro de entrada ou saída...
        catch (IOException e) {
            System.out.println("Erro atendendo a uma conexão !");
        }
    }
}
